/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hserver.h 83 2003-10-07 08:51:02Z pynoos $ */

#ifndef HSERVER_H
#define HSERVER_H

#include "hserverstd.h"
#include "hslist.h"
#include "hslock.h"

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author Hojin Choi
 *  \date Sun Sep 28 11:43:17 2003
 *  \brief application including header
 * 
 */

/* DECLARATION ----------------------------------------------------------------------*/
enum _hs_state
{
	HSS_STATE_BEGIN = 1000,
	HSS_LISTEN,
	HSS_ACCEPT,
	HSS_ONACCEPT,
	HSS_RECV,
	HSS_ONRECV,
	HSS_SEND,
	HSS_ONSENT,
	HSS_CLOSE,
	HSS_ONCLOSE,
	HSS_ONERROR,
	HSS_STATE_END
};

enum _hs_type
{
	HS_FORK = 1,
	HS_THREAD = 2,
	HS_PREFORK = 4,
	HS_NONBLOCKIO = 8,
	HS_ASYNCIO = 16,
	HS_PREFORKED_NONBLOCKIO = HS_PREFORK | HS_NONBLOCKIO,
	HS_THREAD_NONBLOCKIO = HS_THREAD | HS_NONBLOCKIO
};

enum _hs_cntl_type
{
	HS_PREFORKS,
	HS_PREFORK_MIN,
	HS_PREFORK_MAX
};

enum _hshandle_type
{
	HSHANDLE_ONRECV,
	HSHANDLE_ONSENT
};

typedef enum _hs_state hs_state;
typedef struct _hsocket hsocket;
typedef int (*hs_callback) (hsocket *);
typedef   BOOL(*hs_vfunc) (hsocket *);

struct _hsocket
{
	hs_state  state;				   /* CURRENT STATE */
	int       fd;					   /* SOCKET */
	BOOL      interesting;				   /* SELECTED FOR ONACCEPT, ONRECV, ONCLOSE */
	int       errnum;				   /* ONERROR */

	          hs_vfunc(*vft)[2];
	hs_callback callback;
};

typedef struct _hserver_engine hserver_engine;

struct _hserver_engine
{
	int       nType;
	int       nInterestings;
	int       nMaxProcesses;
	int       nMinProcesses;
	int       nCurrProcesses;
	int       nSockets;
	hslist_t  lstSockets;
	BOOL      bTerminated;
	BOOL      bEventDriven;

	hslock   *acceptLock;
	hslock_vft *lock_vft;

	          hs_vfunc(*sock_vft)[2];
	          BOOL(*prefork_maintainer) (hserver_engine *);
	          BOOL(*spawn_child) (SOCKET);
	          BOOL(*do_task) (hsocket *);
};

extern hserver_engine *g_pEngine;

typedef   BOOL(*HSINIT_TYPE) (hserver_engine *);

/* EXPORTED VARIABLES ---------------------------------------------------------------*/

extern HSINIT_TYPE HSINIT_FORK;
extern HSINIT_TYPE HSINIT_THREAD;
extern HSINIT_TYPE HSINIT_PREFORK;
extern HSINIT_TYPE HSINIT_NONBLOCKIO;
extern HSINIT_TYPE HSINIT_ASYNCIO;
extern HSINIT_TYPE HSINIT_PREFORKED_NONBLOCKIO;
extern HSINIT_TYPE HSINIT_THREAD_NONBLOCKIO;

/* EXPORTED FUNCTIONS ---------------------------------------------------------------*/

hsocket  *hsocket_server(const char *ip, int port);
hsocket  *hsocket_new();
void      hsocket_delete(hsocket * s);

BOOL      hs_init(HSINIT_TYPE);
void      hs_task_install(hsocket * s, hs_callback pfCallBack);
void      hs_run();
void      hs_destroy();

/* INLINE FUNCTIONS -----------------------------------------------------------------*/

#endif
/*
 * History
   $Log$
   Revision 1.4  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

   Revision 1.3  2003/09/28 05:23:22  pynoos
   Copy from model4.c

   Revision 1.2  2003/09/28 05:00:04  pynoos
   Move define

   Revision 1.1  2003/09/28 03:55:17  pynoos
   Add first draft

 */
