/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hserver_fork.c 83 2003-10-07 08:51:02Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author Hojin Choi
 *  \brief Fork-mode engine
 * 
 */

/* FORWARD DECLARATION --------------------------------------------------------------*/
#include "hserverstd.h"
#include "hserver_fork.h"
#include "hslog.h"

static BOOL hs_init_fork(hserver_engine *);
HSINIT_TYPE HSINIT_FORK = &hs_init_fork;

static BOOL hsvft_fork_accept(hsocket *);

hs_vfunc  hsvft_fork[] = {
	hsvft_fork_accept,
	0
};

/* LOCAL DATA -----------------------------------------------------------------------*/

/* FUNCTION DEFINITION --------------------------------------------------------------*/

static BOOL hs_init_fork(hserver_engine * pEngine)
{
	pEngine->nType = HS_FORK;
	pEngine->sock_vft = &hsvft_fork;
	return TRUE;
}

static BOOL hsvft_fork_accept(hsocket * pSockServer)
{
	hs_callback callback;
	BOOL      bSuccessFork = FALSE;
	pid_t     pid;
	struct sockaddr_in addr;
	int       size = sizeof addr;
	hsocket   sockWorker;

	sockWorker.fd = accept(pSockServer->fd, (struct sockaddr *) &addr, &size);
	sockWorker.state = HSS_RECV;

	/* Try to fork a process */
	while (!bSuccessFork)
	{
		switch ((pid = fork()))
		{
		case -1:
			hserrlog("fork");
			sleep(1);
			continue;
		case 0:
			sockWorker.state = HSS_RECV;
			bSuccessFork = TRUE;
			break;
		default:
			hslog(2, "worker forked [pid:%d]", pid);
			close(sockWorker.fd);
			return TRUE;
		}
	}
	/* end fork; child process will go on */
	callback = pSockServer->callback;
	(*callback) (&sockWorker);
	shutdown(sockWorker.fd, 2);
	hslog(2, "wait peer close");
	sleep(1);
	close(sockWorker.fd);
	exit(0);
	return FALSE;
}

#ifndef NDEBUG
static char id[] __UNUSED__ = "$Id: hserver_fork.c 83 2003-10-07 08:51:02Z pynoos $";
#endif
/*
 * History
   $Log$
   Revision 1.1  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

 */
