/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hserver_prefork.c 83 2003-10-07 08:51:02Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author
 *  \brief
 * 
 */

/* FORWARD DECLARATION --------------------------------------------------------------*/
#include "hserverstd.h"
#include "hslog.h"
#include "hslock_sem.h"
#include "hserver.h"
#include "hserver_prefork.h"

static BOOL hs_init_prefork(hserver_engine * pEngine);
static BOOL hs_prefork_maintainer(hserver_engine * pEngine);
HSINIT_TYPE HSINIT_PREFORK = &hs_init_prefork;

static BOOL hsvft_prefork_accept(hsocket *);

hs_vfunc  hsvft_prefork[] = {
	hsvft_prefork_accept,
	0
};

/* LOCAL DATA -----------------------------------------------------------------------*/

/* FUNCTION DEFINITION --------------------------------------------------------------*/

static BOOL hs_init_prefork(hserver_engine * pEngine)
{
	pEngine->nType = HS_PREFORK;
	pEngine->sock_vft = &hsvft_prefork;
	pEngine->prefork_maintainer = &hs_prefork_maintainer;
	pEngine->lock_vft = g_lock_semaphore;
	pEngine->nMaxProcesses = 1;
	pEngine->nMinProcesses = 1;
	return TRUE;
}

static BOOL keep_processes(hserver_engine * pEngine)
{
	while (!pEngine->bTerminated && pEngine->nCurrProcesses < pEngine->nMinProcesses)
	{
		pid_t     pid;
		switch ((pid = fork()))
		{
		case -1:
			hserrlog("fork");
			sleep(1);
			continue;
		case 0:
			/* Long return... */
			return FALSE;
		default:
			pEngine->nCurrProcesses++;
			hslog(2, "forked [pid:%d]", pid);
		}
	}
	return TRUE;
}

BOOL hs_prefork_maintainer(hserver_engine * pEngine)
{
	while (!pEngine->bTerminated)
	{
		if (!keep_processes(pEngine))
		{
			return FALSE;
		}
		sleep(300);
	}
	return TRUE;
}

static BOOL hsvft_prefork_accept(hsocket * pSockServer)
{
	hs_callback callback;
	struct sockaddr_in addr;
	int       size = sizeof addr;
	hsocket   sockWorker;

	sockWorker.fd = accept(pSockServer->fd, (struct sockaddr *) &addr, &size);
	sockWorker.state = HSS_RECV;
	/* end fork; child process will go on */
	callback = pSockServer->callback;
	(*callback) (&sockWorker);
	shutdown(sockWorker.fd, 2);
	hslog(2, "wait peer close");
	close(sockWorker.fd);

	/* Return with FALSE will escape descriptor check routine */
	return FALSE;
}

#ifndef NDEBUG
static char id[] __UNUSED__ = "$Id: hserver_prefork.c 83 2003-10-07 08:51:02Z pynoos $";
#endif
/*
 * History
   $Log$
   Revision 1.1  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

 */
