/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hserver_thread.c 83 2003-10-07 08:51:02Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author Hojin Choi
 *  \brief Thread creator
 * 
 */

/* FORWARD DECLARATION --------------------------------------------------------------*/
#include "hserverstd.h"
#include "hserver.h"
#include "hserver_fork.h"
#include "hslog.h"

static BOOL hs_init_thread(hserver_engine *);
HSINIT_TYPE HSINIT_THREAD = &hs_init_thread;

static BOOL hsvft_thread_accept(hsocket *);

hs_vfunc  hsvft_fork[] = {
	hsvft_thread_accept,
	0
};

typedef struct _hsthread_pv
{
	pthread_t thrID;
	hsocket  *sockServer;
	hsocket  *sock;
}
hsthread_pv;

/* LOCAL DATA -----------------------------------------------------------------------*/

/* FUNCTION DEFINITION --------------------------------------------------------------*/

static BOOL hs_init_thread(hserver_engine * pEngine)
{
	pEngine->nType = HS_THREAD;
	pEngine->sock_vft = &hsvft_fork;
	return TRUE;
}

static void *hserver_thread_entrance(void *pv)
{
	hs_callback callback;
	hsocket  *pSockWorker = ((hsthread_pv *) pv)->sock;

	callback = ((hsthread_pv *) pv)->sockServer->callback;
	(*callback) (pSockWorker);
	shutdown(pSockWorker->fd, 2);
	hslog(2, "wait peer close");
	sleep(1);
	close(pSockWorker->fd);
	hsocket_delete(pSockWorker);
	free(pv);
	pthread_exit(0);
	return 0;
}

static BOOL hsvft_thread_accept(hsocket * pSockServer)
{
	BOOL      bSuccessSpawn = FALSE;
	struct sockaddr_in addr;
	int       size = sizeof addr;
	hsthread_pv *pv;
	pthread_attr_t attrThread;

	pv = (hsthread_pv *) malloc(sizeof(*pv));
	if (!pv)
	{
		TODO("Graceful socket error needed.");
		return FALSE;
	}

	pv->sock = hsocket_new();
	if (!pv->sock)
	{
		TODO("Graceful socket error needed.");
		return FALSE;
	}

	pv->sockServer = pSockServer;
	pv->sock->fd = accept(pSockServer->fd, (struct sockaddr *) &addr, &size);
	pv->sock->state = HSS_RECV;

	/* Try to create a thread */

	pthread_attr_init(&attrThread);
	pthread_attr_setdetachstate(&attrThread, PTHREAD_CREATE_DETACHED);

	while (!bSuccessSpawn)
	{
		switch (pthread_create(&pv->thrID, &attrThread, hserver_thread_entrance, (void *) pv))
		{
		case 0:
			hslog(2, "thread created [threadid:%d]", pv->thrID);
			bSuccessSpawn = TRUE;
			break;
		default:
			hserrlog("pthread_create");
			sleep(1);
			continue;
		}
	}

	/* pv's owner ship will be passed to the new thread so, we don't free it */
	pthread_attr_destroy(&attrThread);
	return FALSE;
}

#ifndef NDEBUG
static char id[] __UNUSED__ = "$Id: hserver_thread.c 83 2003-10-07 08:51:02Z pynoos $";
#endif
/*
 * History
   $Log$
   Revision 1.1  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

 */
