/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hserverstd.h 81 2003-09-28 23:05:53Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author Hojin Choi
 *  \brief Standard include header
 * 
 * This header includes system header by config.h depency results.
 */

#ifndef HSERVERSTD_H
#define HSERVERSTD_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
#include "hsconfig.h"
#endif

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_REGEX_H
#include <regex.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/stat.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_IPC_H
#include <sys/ipc.h>
#endif

#ifdef HAVE_SYS_SEM_H
#include <sys/sem.h>
#endif

#ifdef HAVE_SYS_UN_H
#include <sys/un.h>
#endif

#define queue __ahn_queue
/* This hack is avoid typename collision in Solaris 2.5.1. gcc 2.8.1
   queue is dclared in below two headers
   sys/stream.h is included in netinet/in.h
   sys/stream.h
   stl_queue.h
*/
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_ASSERT_H
#include <assert.h>
#endif

#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#ifdef HAVE_UTIME_H
#include <utime.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_NETINET_TCP_H
#include <netinet/tcp.h>
#endif

#ifdef HAVE_SYS_UN_H
#include <sys/un.h>
#endif

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif

#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif

#ifdef HAVE_LINUX_LIMITS_H
#include <linux/limits.h>
#endif

#ifdef HAVE_SYS_ERRNO_H
#include <sys/errno.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* This Macro defined before netinet/in.h for avoid porting issue in solaris 2.5.1 and gcc 2.8.1, see above */
#undef queue

#ifndef __cplusplus
#define BOOL	int
#define FALSE	0
#define TRUE	1
#endif

#ifndef SOCKET
#define SOCKET int
#endif

#ifdef __cplusplus
#include <string>
#include <set>
#include <map>
#include <vector>
#endif

union semun
{
	int       val;
	struct semid_ds *buf;
	unsigned short int *array;
	struct seminfo *__buf;
};

/* DECLARATION ----------------------------------------------------------------------*/

#ifdef __GNUC__
#	define __UNUSED__ 	__attribute__((unused))
#else
#	define __UNUSED__
#endif
/* __UNUSED__ macro is for unused variable declaration, except in function argument.*/

#ifndef NDEBUG
#	define __RELEASE_UNUSED__
#else
#	define __RELEASE_UNUSED__	__UNUSED__
#endif
/* __RELEASE_UNUSED__ is for assert check to avoid warning.*/
/* e.g. */
/* int __RELEASE_UNUSED__ ret = func();*/
/* assert( ret );*/

/* from "glibc/include/libc-symbols.h" */
#ifdef __GNUC__
#define link_warning(symbol, msg) \
  static const char __evoke_link_warning_##symbol[]	\
    __attribute__ ((section (".gnu.warning." #symbol "\n\t#"))) = msg;
#else
#define link_warning(symbol, msg)
#endif
/* until here "glibc/include/libc-symbols.h" */

#define LARGE_BUFFER_SIZE (4096)
#define BUFFER_SIZE 	  (1024)
#define SMALL_BUFFER_SIZE (128)

#define TODO_ERR(x)       do { perror( "ERROR" ); assert( 0 && (x) ); } while(0)
#define TODO(x)           assert( 0 && (x) )

#ifdef __cplusplus
#define FOR_EACH( it, cont ) for( it=(cont).begin(); (it) != (cont).end(); (it)++ )
#define FOR_EACH_NOINC( it, cont ) for( it=(cont).begin(); (it) != (cont).end(); /*no-inc*/)

typedef vector <string >StringVector;
typedef set <string >StringSet;
typedef map <string, string >StringStringMap;

using namespace std;
#endif

/* EXPORTED VARIABLES ---------------------------------------------------------------*/

/* EXPORTED FUNCTIONS ---------------------------------------------------------------*/

/* INLINE FUNCTIONS -----------------------------------------------------------------*/

#endif /*HSERVERSTD_H */
/*
 * History
   $Log$
   Revision 1.7  2003/09/28 23:05:53  pynoos
   Remove define macro for declaring union semun

   Revision 1.6  2003/09/28 03:55:17  pynoos
   Add first draft

   Revision 1.5  2003/09/22 16:33:24  pynoos
   Add semaphore related headers

   Revision 1.4  2003/09/16 15:16:20  pynoos
   Add some macroes

   Revision 1.3  2003/08/31 17:12:37  pynoos
   Added system header includes

   Revision 1.2  2003/08/30 07:39:12  pynoos
   New!

   Revision 1.1  2003/08/30 07:36:56  pynoos
   New added test environment

 */
