/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hslock_sem.c 90 2003-10-07 15:18:03Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author
 *  \brief
 * 
 */

/* FORWARD DECLARATION --------------------------------------------------------------*/
#include "hserverstd.h"
#include "hslog.h"
#include "hslock_sem.h"

static hslock *hslock_init(int *pbTermination);
static BOOL hslock_close(hslock * plock);
static BOOL hslock_hold(hslock * plock);
static BOOL hslock_release(hslock * plock);

/* LOCAL DATA -----------------------------------------------------------------------*/

hslock_vft _lock_semaphore = {
	hslock_init,
	hslock_hold,
	hslock_release,
	hslock_close
};

hslock_vft *g_lock_semaphore = &_lock_semaphore;

typedef struct _hslock_semaphore_ctx
{
	int       ipc_id;
	int      *pbTermination;

}
hslock_semaphore_ctx;

#define HSIPCID(l) (((hslock_semaphore_ctx*)l)->ipc_id)
#define HSTERMINATED(l) (((hslock_semaphore_ctx*)l)->pbTermination)

/* FUNCTION DEFINITION --------------------------------------------------------------*/

static BOOL hslock_close(hslock * plock)
{
	BOOL      ret;
	assert(plock);
	ret = semctl(HSIPCID(plock), 0, IPC_RMID, 0);
	free(plock);
	return ret;
}

static hslock *hslock_init(int *pbTermination)
{
	hslock_semaphore_ctx *plock;
	union semun semval;

	assert(pbTermination);

	plock = (hslock_semaphore_ctx *) malloc(sizeof(hslock_semaphore_ctx));
	if (!plock)
	{
		return 0;
	}
	plock->ipc_id = semget(IPC_PRIVATE, 1, IPC_CREAT | 0600);
	plock->pbTermination = pbTermination;
	if (HSIPCID(plock) < 0)
	{
		hserrlog("semget");
		free(plock);
		return 0;
	}
	semval.val = 1;
	if (semctl(HSIPCID(plock), 0, SETVAL, semval) < 0)
	{
		hserrlog("semctl");
		free(plock);
		return 0;
	}
	return (hslock *) plock;
}

static BOOL hslock_hold(hslock * plock)
{
	static struct sembuf ops = { 0, -1, SEM_UNDO };

	assert(plock);

	while (0 > semop(HSIPCID(plock), &ops, 1))
	{
		if (errno == EINTR && !HSTERMINATED(plock))
		{
			continue;
		}
		hserrlog("semop");
		return FALSE;
	}
	return TRUE;
}

static BOOL hslock_release(hslock * plock)
{
	static struct sembuf ops = { 0, 1, SEM_UNDO };

	assert(plock);

	while (0 > semop(HSIPCID(plock), &ops, 1))
	{
		if (errno == EINTR && !HSTERMINATED(plock))
		{
			continue;
		}
		hserrlog("semop");
		return FALSE;
	}
	return TRUE;
}

#ifndef NDEBUG
static char id[] __UNUSED__ = "$Id: hslock_sem.c 90 2003-10-07 15:18:03Z pynoos $";
#endif
/*
 * History
   $Log$
   Revision 1.2  2003/10/07 15:18:03  pynoos
   Add assert

   Revision 1.1  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

 */
