/*
 * This is a part of hserver project.                                                
 * You can freely redistribute this source code under BSD license
 */
/* $Id: hslog.c 83 2003-10-07 08:51:02Z pynoos $ */

/* FILE DESCRIPTION -----------------------------------------------------------------*/

/*! \author
 *  \brief
 * 
 */

/* FORWARD DECLARATION --------------------------------------------------------------*/
#include "hserverstd.h"

/* LOCAL DATA -----------------------------------------------------------------------*/

/* FUNCTION DEFINITION --------------------------------------------------------------*/

/*! \brief HServer Log API
 *
 * Main Log API
 *
 * \param level loglevel which can varies 0 or bigger.
 * \param fmt log template usually used in printf.
 * \return 0 : successful
 * \see hserrlog
 */

#ifndef NDEBUG
static char id[] __UNUSED__ = "$Id: hslog.c 83 2003-10-07 08:51:02Z pynoos $";
#endif
#include "hserverstd.h"

#define s_stdlog stderr

static int s_level = -1;

int hslog(int level, const char *fmt, ...)
{
	char      buf[BUFFER_SIZE];
	va_list   marker;
	const char *HSLOGLEVEL;

	if (s_level == -1)
	{
		HSLOGLEVEL = getenv("HSLOGLEVEL");
		if (HSLOGLEVEL)
		{
			s_level = atoi(HSLOGLEVEL);
		}
		if (s_level < 0)
		{
			s_level = 0;
		}
	}

	if (level > s_level)
	{
		return 0;
	}

	va_start(marker, fmt);
	vsnprintf(buf, sizeof buf, fmt, marker);
	strcat(buf, "\n");
	fputs(buf, s_stdlog);
	va_end(marker);
	return 0;
}

/*! \brief Error message logger
 * This function used in logging system error message (concerned with errno)
 * error level sticked to 0.
 * \param fmt format which is used in printf.
 * \return 0 always yet!
 * \see hslog
 */

int hserrlog(const char *fmt, ...)
{
	char      buf[BUFFER_SIZE];
	va_list   marker;
#define MAXERRORMSG 256

	va_start(marker, fmt);
	vsnprintf(buf, sizeof buf - MAXERRORMSG, fmt, marker);

	strcat(buf, " (");
	strcat(buf, strerror(errno));
	strcat(buf, ")");

	hslog(1, "%s", buf);

	va_end(marker);
	return 0;
}

/*
 * History
   $Log$
   Revision 1.2  2003/10/07 08:51:02  pynoos
   First draft of BLOCKED-TYPE series

   Revision 1.1  2003/09/28 05:00:28  pynoos
   Add hslog

   Revision 1.1  2003/09/08 17:27:57  pynoos
   New added

 */
